+{
   locale_version => 0.88,
   entry => <<'ENTRY', # for DUCET v6.1.0
0D82      ; [.223F.0020.0002.0D82][.FFF1.0000.0000.0000] # SINHALA SIGN ANUSVARAYA
0D83      ; [.223F.0020.0002.0D83][.FFF2.0000.0000.0000] # SINHALA SIGN VISARGAYA
0DA5      ; [.224A.0020.0002.0DA5] # SINHALA LETTER TAALUJA SANYOOGA NAAKSIKYAYA
0DA4      ; [.224B.0020.0002.0DA4] # SINHALA LETTER TAALUJA NAASIKYAYA
ENTRY
};
