+{
   locale_version => 0.88,
   entry => <<'ENTRY', # for DUCET v6.1.0
010D      ; [.1615.0020.0002.010D] # LATIN SMALL LETTER C WITH CARON
0063 030C ; [.1615.0020.0002.010D] # LATIN SMALL LETTER C WITH CARON
010C      ; [.1615.0020.0008.010C] # LATIN CAPITAL LETTER C WITH CARON
0043 030C ; [.1615.0020.0008.010C] # LATIN CAPITAL LETTER C WITH CARON
0123      ; [.1698.0020.0002.0123] # LATIN SMALL LETTER G WITH CEDILLA
0067 0327 ; [.1698.0020.0002.0123] # LATIN SMALL LETTER G WITH CEDILLA
0122      ; [.1698.0020.0008.0122] # LATIN CAPITAL LETTER G WITH CEDILLA
0047 0327 ; [.1698.0020.0008.0122] # LATIN CAPITAL LETTER G WITH CEDILLA
0137      ; [.16F5.0020.0002.0137] # LATIN SMALL LETTER K WITH CEDILLA
006B 0327 ; [.16F5.0020.0002.0137] # LATIN SMALL LETTER K WITH CEDILLA
0136      ; [.16F5.0020.0008.0136] # LATIN CAPITAL LETTER K WITH CEDILLA
004B 0327 ; [.16F5.0020.0008.0136] # LATIN CAPITAL LETTER K WITH CEDILLA
013C      ; [.1725.0020.0002.013C] # LATIN SMALL LETTER L WITH CEDILLA
006C 0327 ; [.1725.0020.0002.013C] # LATIN SMALL LETTER L WITH CEDILLA
013B      ; [.1725.0020.0008.013B] # LATIN CAPITAL LETTER L WITH CEDILLA
004C 0327 ; [.1725.0020.0008.013B] # LATIN CAPITAL LETTER L WITH CEDILLA
0146      ; [.1755.0020.0002.0146] # LATIN SMALL LETTER N WITH CEDILLA
006E 0327 ; [.1755.0020.0002.0146] # LATIN SMALL LETTER N WITH CEDILLA
0145      ; [.1755.0020.0008.0145] # LATIN CAPITAL LETTER N WITH CEDILLA
004E 0327 ; [.1755.0020.0008.0145] # LATIN CAPITAL LETTER N WITH CEDILLA
0157      ; [.17D7.0020.0002.0157] # LATIN SMALL LETTER R WITH CEDILLA
0072 0327 ; [.17D7.0020.0002.0157] # LATIN SMALL LETTER R WITH CEDILLA
0156      ; [.17D7.0020.0008.0156] # LATIN CAPITAL LETTER R WITH CEDILLA
0052 0327 ; [.17D7.0020.0008.0156] # LATIN CAPITAL LETTER R WITH CEDILLA
0161      ; [.17FA.0020.0002.0161] # LATIN SMALL LETTER S WITH CARON
0073 030C ; [.17FA.0020.0002.0161] # LATIN SMALL LETTER S WITH CARON
0160      ; [.17FA.0020.0008.0160] # LATIN CAPITAL LETTER S WITH CARON
0053 030C ; [.17FA.0020.0008.0160] # LATIN CAPITAL LETTER S WITH CARON
017E      ; [.1896.0020.0002.017E] # LATIN SMALL LETTER Z WITH CARON
007A 030C ; [.1896.0020.0002.017E] # LATIN SMALL LETTER Z WITH CARON
017D      ; [.1896.0020.0008.017D] # LATIN CAPITAL LETTER Z WITH CARON
005A 030C ; [.1896.0020.0008.017D] # LATIN CAPITAL LETTER Z WITH CARON
ENTRY
};
