+{
   locale_version => 0.88,
# eng doesn't require tailoring
   entry => <<'ENTRY', # for DUCET v6.1.0
00E0      ; [.15D5.0020.0002.00E0] # LATIN SMALL LETTER A WITH GRAVE
0061 0300 ; [.15D5.0020.0002.00E0] # LATIN SMALL LETTER A WITH GRAVE
0061 0340 ; [.15D5.0020.0002.00E0] # LATIN SMALL LETTER A WITH GRAVE
00C0      ; [.15D5.0020.0008.00C0] # LATIN CAPITAL LETTER A WITH GRAVE
0041 0300 ; [.15D5.0020.0008.00C0] # LATIN CAPITAL LETTER A WITH GRAVE
0041 0340 ; [.15D5.0020.0008.00C0] # LATIN CAPITAL LETTER A WITH GRAVE
00E9      ; [.1632.0020.0002.00E9] # LATIN SMALL LETTER E WITH ACUTE
0065 0301 ; [.1632.0020.0002.00E9] # LATIN SMALL LETTER E WITH ACUTE
0065 0341 ; [.1632.0020.0002.00E9] # LATIN SMALL LETTER E WITH ACUTE
00C9      ; [.1632.0020.0008.00C9] # LATIN CAPITAL LETTER E WITH ACUTE
0045 0301 ; [.1632.0020.0008.00C9] # LATIN CAPITAL LETTER E WITH ACUTE
0045 0341 ; [.1632.0020.0008.00C9] # LATIN CAPITAL LETTER E WITH ACUTE
00EB      ; [.1633.0020.0002.00EB] # LATIN SMALL LETTER E WITH DIAERESIS
0065 0308 ; [.1633.0020.0002.00EB] # LATIN SMALL LETTER E WITH DIAERESIS
00CB      ; [.1633.0020.0008.00CB] # LATIN CAPITAL LETTER E WITH DIAERESIS
0045 0308 ; [.1633.0020.0008.00CB] # LATIN CAPITAL LETTER E WITH DIAERESIS
00F1      ; [.1735.0020.0002.00F1] # LATIN SMALL LETTER N WITH TILDE
006E 0303 ; [.1735.0020.0002.00F1] # LATIN SMALL LETTER N WITH TILDE
00D1      ; [.1735.0020.0008.00D1] # LATIN CAPITAL LETTER N WITH TILDE
004E 0303 ; [.1735.0020.0008.00D1] # LATIN CAPITAL LETTER N WITH TILDE
00F3      ; [.1757.0020.0002.00F3] # LATIN SMALL LETTER O WITH ACUTE
006F 0301 ; [.1757.0020.0002.00F3] # LATIN SMALL LETTER O WITH ACUTE
006F 0341 ; [.1757.0020.0002.00F3] # LATIN SMALL LETTER O WITH ACUTE
00D3      ; [.1757.0020.0008.00D3] # LATIN CAPITAL LETTER O WITH ACUTE
004F 0301 ; [.1757.0020.0008.00D3] # LATIN CAPITAL LETTER O WITH ACUTE
004F 0341 ; [.1757.0020.0008.00D3] # LATIN CAPITAL LETTER O WITH ACUTE
ENTRY
};
