+{
   locale_version => 0.88,
# in cldr test/lt.xml why I-dot-acute > I-dot though i-dot-acute < i-dot ?
   entry => <<'ENTRY', # for DUCET v6.1.0
0049 0307 ; [.16B2.0020.0008.0049][.0000.0052.0002.0307] # <LATIN CAPITAL LETTER I, COMBINING DOT ABOVE>
0307 0300 ; [.0000.0035.0002.0300] # <COMBINING DOT ABOVE, COMBINING GRAVE ACCENT>
0307 0301 ; [.0000.0032.0002.0301] # <COMBINING DOT ABOVE, COMBINING ACUTE ACCENT>
0307 0303 ; [.0000.004E.0002.0303] # <COMBINING DOT ABOVE, COMBINING TILDE>
0105      ; [.15D4.0021.0002.0105] # LATIN SMALL LETTER A WITH OGONEK
0061 0328 ; [.15D4.0021.0002.0105] # LATIN SMALL LETTER A WITH OGONEK
0104      ; [.15D4.0021.0008.0104] # LATIN CAPITAL LETTER A WITH OGONEK
0041 0328 ; [.15D4.0021.0008.0104] # LATIN CAPITAL LETTER A WITH OGONEK
010D      ; [.1603.0020.0002.010D] # LATIN SMALL LETTER C WITH CARON
0063 030C ; [.1603.0020.0002.010D] # LATIN SMALL LETTER C WITH CARON
010C      ; [.1603.0020.0008.010C] # LATIN CAPITAL LETTER C WITH CARON
0043 030C ; [.1603.0020.0008.010C] # LATIN CAPITAL LETTER C WITH CARON
0119      ; [.1631.0021.0002.0119] # LATIN SMALL LETTER E WITH OGONEK
0065 0328 ; [.1631.0021.0002.0119] # LATIN SMALL LETTER E WITH OGONEK
0118      ; [.1631.0021.0008.0118] # LATIN CAPITAL LETTER E WITH OGONEK
0045 0328 ; [.1631.0021.0008.0118] # LATIN CAPITAL LETTER E WITH OGONEK
0117      ; [.1631.0022.0002.0117] # LATIN SMALL LETTER E WITH DOT ABOVE
0065 0307 ; [.1631.0022.0002.0117] # LATIN SMALL LETTER E WITH DOT ABOVE
0116      ; [.1631.0022.0008.0116] # LATIN CAPITAL LETTER E WITH DOT ABOVE
0045 0307 ; [.1631.0022.0008.0116] # LATIN CAPITAL LETTER E WITH DOT ABOVE
012F      ; [.16B2.0021.0002.012F] # LATIN SMALL LETTER I WITH OGONEK
0069 0328 ; [.16B2.0021.0002.012F] # LATIN SMALL LETTER I WITH OGONEK
012E      ; [.16B2.0021.0008.012E] # LATIN CAPITAL LETTER I WITH OGONEK
0049 0328 ; [.16B2.0021.0008.012E] # LATIN CAPITAL LETTER I WITH OGONEK
0079      ; [.16B2.0022.0002.0079] # LATIN SMALL LETTER Y
0059      ; [.16B2.0022.0008.0059] # LATIN CAPITAL LETTER Y
0161      ; [.17D9.0020.0002.0161] # LATIN SMALL LETTER S WITH CARON
0073 030C ; [.17D9.0020.0002.0161] # LATIN SMALL LETTER S WITH CARON
0160      ; [.17D9.0020.0008.0160] # LATIN CAPITAL LETTER S WITH CARON
0053 030C ; [.17D9.0020.0008.0160] # LATIN CAPITAL LETTER S WITH CARON
0173      ; [.181B.0021.0002.0173] # LATIN SMALL LETTER U WITH OGONEK
0075 0328 ; [.181B.0021.0002.0173] # LATIN SMALL LETTER U WITH OGONEK
0172      ; [.181B.0021.0008.0172] # LATIN CAPITAL LETTER U WITH OGONEK
0055 0328 ; [.181B.0021.0008.0172] # LATIN CAPITAL LETTER U WITH OGONEK
016B      ; [.181B.0022.0002.016B] # LATIN SMALL LETTER U WITH MACRON
0075 0304 ; [.181B.0022.0002.016B] # LATIN SMALL LETTER U WITH MACRON
016A      ; [.181B.0022.0008.016A] # LATIN CAPITAL LETTER U WITH MACRON
0055 0304 ; [.181B.0022.0008.016A] # LATIN CAPITAL LETTER U WITH MACRON
017E      ; [.187B.0020.0002.017E] # LATIN SMALL LETTER Z WITH CARON
007A 030C ; [.187B.0020.0002.017E] # LATIN SMALL LETTER Z WITH CARON
017D      ; [.187B.0020.0008.017D] # LATIN CAPITAL LETTER Z WITH CARON
005A 030C ; [.187B.0020.0008.017D] # LATIN CAPITAL LETTER Z WITH CARON
ENTRY
};
